#!/usr/bin/env perl

use strict;
use warnings;
use constant VERSION => '0.2';
use Getopt::Long;

unless(scalar @ARGV) {
  print 'ghc-libs.pl script v'.VERSION."\n";
  print "(c) Alexandr A Alexeev 2012 | http://eax.me/\n";
  print "Usage: $0 [--dump-libraries] lib1 [lib2 ...]\n";
  exit 1;
}

my $libdir = defined $ENV{HASKELL_LIB}
  ? $ENV{HASKELL_LIB}
  : `ghc --print-libdir`;
die "ERROR: `ghc --print-libdir` - error code $?\n" if($?);
chomp($libdir);

my $ver = `ghc --version`;
die "ERROR: `ghc --version` - error code $?\n" if($?);
chomp($ver);
my ($ghcver) = $ver =~ / version (\d+\.\d+\.\d+)$/i;
die "ERROR: invalid `ghc --version` output:\n----\n$ver\n----\n"
  unless defined $ghcver;

my $home = defined $ENV{HASKELL_HOME}
      ? $ENV{HASKELL_HOME}
      : $ENV{HOME}."/.cabal/lib"; 

my @search = (
    "$home/*/ghc-$ghcver/libHS*.a",
    "/usr/local/lib/*/ghc-$ghcver/libHS*.a",
    "$libdir/*/libHS*.a",
  );

my %lib_hash;
my %opts;
GetOptions(\%opts, '--dump-libraries');

print "-L$libdir -lHSrts ";

for my $glob (@search) {
  for my $file (glob $glob) {
		# warn "file = $file\n";
    my ($path, $lib, $ver) = $file =~ m#^(.*?)/libHS(.*?)-([\d\.]*?).a$#;
    next unless
      defined $path &&
      defined $lib &&
      defined $ver;
    $lib_hash{$lib} = "-L$path -lHS$lib-$ver " unless defined $lib_hash{$lib};
  }
}

if($opts{'dump-libraries'}) {
  use Data::Dumper;
  warn Dumper(\%lib_hash);
}

for my $lib (@ARGV) {
  if(defined $lib_hash{$lib}) {
    print $lib_hash{$lib};
  } else {
    die "ERROR: unknown library '$lib'\n";
  }
}

print "\n";
